#ifdef HAVE_OPENSSL
#include <openssl/evp.h>
#include <openssl/bio.h>
#include <openssl/md5.h>
#else
#include <gcrypt.h>
#endif

#ifndef __WIN32
#include <syslog.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/in_systm.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <sys/wait.h>
#else
#define LOG_CRIT 0
#define LOG_INFO 0
#include <winsock2.h>
#endif
#include <zlib.h>

#ifdef _WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
typedef SOCKET netsock;
typedef SOCKADDR netsockaddr;
typedef SOCKADDR_IN netsockaddr_in;
typedef int netsocklen_t;
#else
typedef socklen_t netsock_len;
typedef int netsock;
typedef struct sockaddr netsockaddr;
typedef struct sockaddr_in netsockaddr_in;
typedef socklen_t netsocklen_t;
#endif

#include "log.h"
#include "thread.h"

struct Packet {
    unsigned char *data;
    int len;
};
struct crypt {
    unsigned char *data;
    int len;
};
struct compression {
    unsigned char *data;
    int len;
};
struct keys {
    int version;
    unsigned char key[18];
    unsigned char iv[18];
    char ciphername[255];
#ifdef HAVE_OPENSSL
    const EVP_CIPHER *cipher;
#else
    int cipher;
    int mode;
#endif
};
struct options {
    char host[255];
    int port_listen;
    int port_forward;
    int server;
    int daemon;
    int proto;
    int packet_size;
    int comp;
    int comp_level;
    struct keys key;
    char cipher[255];
#ifndef HAVE_OPENSSL
    char mode[25];
#endif
};
struct Network_timer {
    clock_t start;
    clock_t now;
    float bps;
    float total;
};
struct Network_rate {
    struct Network_timer send;
    struct Network_timer recv;
};
struct Network {
    netsockaddr_in addr;
    netsock sockfd;
    netsocklen_t len;
    struct Network_rate rate;
};
struct Compress {
    z_stream zip;
    z_stream unzip;
};
#ifdef HAVE_OPENSSL
typedef EVP_CIPHER_CTX crypto_ctx;
#else
typedef gcry_cipher_hd_t crypto_ctx;
#endif

struct Ctunnel {
#ifdef HAVE_OPENSSL
    crypto_ctx *ectx;
    crypto_ctx *dctx;
#else
    crypto_ctx ectx;
    crypto_ctx dctx;
#endif
    struct Network *net_cli;
    struct Network *net_srv;
    struct Compress comp;
    struct options opt;
    int srv_sockfd;
    int clisockfd;
    int srvsockfd;
    int id;
};

pthread_mutex_t mutex;

#define KEYFILE "/.passkey"
#define PACKET_SIZE 2048 
#define MAX_THREADS 2048

int threads[MAX_THREADS];
