/*
 * Ctunnel - Cyptographic tunnel.
 * Copyright (C) 2008-2011 Jess Mahan <ctunnel@nardcore.org>
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "config.h"
#include "ctunnel.h"
#include "comp.h"
#include "crypto.h"


struct Network *udp_bind(int port)
{
#ifdef _WIN32
    WSADATA wsadata;
    int error;
    error = WSAStartup(MAKEWORD(2,0), &wsadata);
    if (error != NO_ERROR) {
	ctunnel_log(stderr, LOG_CRIT, "Error WSAStartup()\n");
	exit(1);
    }
#endif
    struct Network *udp = calloc(1, sizeof(struct Network));

    udp->sockfd = socket(AF_INET, SOCK_DGRAM, 0);

    udp->addr.sin_family = AF_INET;
    udp->addr.sin_addr.s_addr = INADDR_ANY;
    udp->addr.sin_port = htons(port);
    udp->len = sizeof(netsockaddr_in);

    if (bind(udp->sockfd, (netsockaddr *) &udp->addr, udp->len) == -1) {
	ctunnel_log(stderr, LOG_CRIT, "UDP Bind %s (port %d)", strerror(errno), port);
	exit(1);
    }
    return udp;
}

struct Network *udp_connect(char *ip, int port)
{
    struct Network *udp = calloc(1, sizeof(struct Network));
    struct hostent *ht;
    char host[255] = "";

    if (!isdigit(ip[0])) {
        if ((ht = gethostbyname(ip))) {
            memset(host, 0x00, 255);
            snprintf(host, 32, "%s",
                     inet_ntoa(*(struct in_addr *) ht->h_addr_list[0]));
        } else {
	    ctunnel_log(stderr, LOG_CRIT, "Cannot resolv %s", ip);
	    exit(1);
        }
    } else {
        snprintf(host, 254, "%s", ip);
    }

    udp->sockfd = socket(AF_INET, SOCK_DGRAM, 0);
    udp->addr.sin_family = AF_INET;
    udp->addr.sin_addr.s_addr = inet_addr(host);
    udp->addr.sin_port = htons(port);
    udp->len = sizeof(netsockaddr_in);

    fprintf(stdout, "UDP Connect to %s:%d\n", host, port);

    if (udp->sockfd < 0) {
	ctunnel_log(stderr, LOG_CRIT, "UDP Connect() %s", strerror(errno));
        exit(1);
    }

    return udp;
}

netsock tcp_listen(int port)
{
#ifdef _WIN32
    WSADATA wsadata;
    int error;
    error = WSAStartup(MAKEWORD(2,2), &wsadata);
    if (error != NO_ERROR) {
	ctunnel_log(stderr, LOG_CRIT, "Error WSAStartup()\n");
	exit(1);
    }
    SOCKADDR_IN sin;
#else
    struct sockaddr_in sin;
#endif
    netsock sockfd;
    int x = 1;

    sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
#ifdef _WIN32
    setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *) &x, sizeof(x));
#else
    setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &x, sizeof(x));
#endif
    if (sockfd == -1) {
        perror("socket()");
        exit(1);
    }

    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    sin.sin_port = htons(port);

    if (bind(sockfd, (netsockaddr *) &sin, sizeof(sin)) == -1) {
        perror("bind()");
        exit(1);
    }
    if (listen(sockfd, 8) == -1) {
        perror("listen()");
        exit(1);
    }
    return sockfd;
}
netsock tcp_connect(char *ip, int port)
{
    netsockaddr_in address;
    netsock clifd;
    int servfd;
    struct hostent *ht;
    char host[255] = "";

    clifd = socket(AF_INET, SOCK_STREAM, 0);
    if (clifd < 0) {
        perror("socket");
	exit(1);
    }

    if (!isdigit(ip[0])) {
        ht = gethostbyname(ip);
        memset(host, 0x00, 255);
        snprintf(host, 32, "%s",
                 inet_ntoa(*(struct in_addr *) ht->h_addr_list[0]));
    } else {
        snprintf(host, 254, "%s", ip);
    }

    address.sin_family = AF_INET;
    address.sin_addr.s_addr = inet_addr(host);
    address.sin_port = htons(port);

    servfd = connect(clifd, (netsockaddr *) &address, sizeof(address));

    if (servfd < 0) {
        perror("connect");
	return servfd;
    }

    return clifd;
}
int net_read(struct options opt, struct Network *net, char *data, int len)
{
    int ret;

    if (opt.proto == 0) {
        ret = (int) recv(net->sockfd, (char *) data, len, 0);
    } else {
        ret = recvfrom(net->sockfd, data, opt.packet_size,
                       0, (netsockaddr *) &net->addr, &net->len);
    }

    net->rate.recv.total += ret;

    return ret;
}
int net_write(struct options opt, struct Network *net, char *data, int len)
{
    int ret;

    if (opt.proto == 0)
	ret = (int) send(net->sockfd, (char *) data, len, 0);
    else
	ret = sendto(net->sockfd, data, len, 0, (netsockaddr *) &net->addr,
	             (netsocklen_t) sizeof(netsockaddr));

    net->rate.send.total += ret;

    return ret;
}
void net_close(struct Network *net)
{
#ifdef _WIN32
    closesocket(net->sockfd);
#else
    close(net->sockfd);
#endif
}
